@@ -12,7 +12,7 @@ from functools import reduce
 from typing import Iterator, Optional
 from enum import IntEnum
 
-envvar_re = re.compile(r'^([A-Za-z_0-9]+)\s*=\s*(.*)$')
+ENVVAR_RE = re.compile(r'^([A-Za-z_0-9]+)\s*=\s*(.*)$')
 
 MINUTES_SET = list(range(0, 60))
 HOURS_SET = list(range(0, 24))
@@ -24,10 +24,8 @@ KSH_SHELLS = ['/bin/sh', '/bin/dash', '/bin/ksh', '/bin/bash', '/usr/bin/zsh']
 REBOOT_FILE = '/run/crond.reboot'
 
 USE_LOGLEVELMAX = "@use_loglevelmax@"
-RANDOMIZED_DELAY = "@randomized_delay@" == "True"
 USE_RUNPARTS = "@use_runparts@" == "True"
-PERSISTENT = "@persistent@" == "True"
-LIBDIR = "@libdir@"
+BOOT_DELAY = "@libexecdir@/systemd-cron/boot_delay"
 STATEDIR = "@statedir@"
 
 SELF = os.path.basename(sys.argv[0])
@@ -43,17 +41,32 @@ PART2TIMER = {
 
 CROND2TIMER = {
     'ntpsec': 'ntpsec-rotate-stats',
+    'ntpsec-ntpviz': 'ntpviz-daily',
     'sysstat': 'sysstat-collect',
 }
 
 TARGET_DIR = '/tmp'
 
-for pgm in ('/usr/sbin/sendmail', '/usr/lib/sendmail'):
-    if os.path.exists(pgm):
-        HAS_SENDMAIL = True
-        break
-else:
-    HAS_SENDMAIL = False
+UPTIME = None
+
+def which(exe:str, paths:Optional[str]=None) -> Optional[str]:
+    if paths is None:
+        paths = os.environ.get('PATH', '/usr/bin:/bin')
+    for path in paths.split(os.pathsep):
+        try:
+            abspath = os.path.join(path, exe)
+            statbuf = os.stat(abspath)
+        except:
+            continue
+        if stat.S_IMODE(statbuf.st_mode) & 0o111:
+            return abspath
+
+    return None
+
+HAS_SENDMAIL = bool(which('sendmail', '/usr/sbin:/usr/lib'))
+
+def systemd_bool(string:str) -> bool:
+    return string.lower() in ['yes', 'true', '1']
 
 class Log(IntEnum):
     EMERG = 0
@@ -95,7 +108,6 @@ class Job:
     execstart:str
     scriptlet:str
     valid:bool
-    standardoutput:Optional[str]
     testremoved:Optional[str]
 
     def __init__(self, filename:str, line:str) -> None:
@@ -114,7 +126,6 @@ class Job:
         self.command = []
         self.valid = True
         self.batch = False
-        self.standardoutput = None
         self.testremoved = None
         self.period = ''
         self.timespec_minute = []
@@ -128,6 +139,9 @@ class Job:
     def log(self, priority:int, message:str) -> None:
         log(priority, '%s in %s:%s' % (message, self.filename, self.line))
 
+    def which(self, pgm) -> Optional[str]:
+        return which(pgm, self.environment.get('PATH'))
+
     def decode_environment(self, default_persistent:bool) -> None:
         '''decode some environment variables that influence
            the behaviour of systemd-cron itself'''
@@ -136,7 +150,7 @@ class Job:
             self.shell = self.environment['SHELL']
 
         if 'PERSISTENT' in self.environment:
-            self.persistent = self.environment['PERSISTENT'].lower() in ['yes', 'true', '1']
+            self.persistent = systemd_bool(self.environment['PERSISTENT'])
             del self.environment['PERSISTENT']
         else:
             self.persistent = default_persistent
@@ -167,7 +181,7 @@ class Job:
                 self.log(Log.WARNING, 'invalid DELAY')
 
         if 'BATCH' in self.environment:
-            self.batch = self.environment['BATCH'].lower() in ['yes','true','1']
+            self.batch = systemd_bool(self.environment['BATCH'])
             del self.environment['BATCH']
 
     def parse_anacrontab(self) -> None:
@@ -202,7 +216,7 @@ class Job:
             else:
                 self.user = os.getlogin()
 
-            pgm = which(self.command[0])
+            pgm = self.which(self.command[0])
             if pgm:
                 self.command[0] = pgm
             self.execstart = ' '.join(self.command)
@@ -291,7 +305,7 @@ class Job:
         if self.shell not in KSH_SHELLS:
             return
 
-        pgm = which(self.command[0])
+        pgm = self.which(self.command[0])
         if pgm and pgm != self.command[0]:
             self.command[0] = pgm
 
@@ -299,12 +313,10 @@ class Job:
             self.command[-2] == '>' and
             self.command[-1] == '/dev/null'):
             self.command = self.command[0:-2]
-            self.standardoutput = '/dev/null'
 
         if (len(self.command) >= 2 and
             self.command[-1] == '>/dev/null'):
             self.command = self.command[0:-1]
-            self.standardoutput = '/dev/null'
 
         if (len(self.command) == 6 and
             self.command[0] == '[' and
@@ -487,16 +499,15 @@ class Job:
         if USE_LOGLEVELMAX != 'no':
             lines.append('LogLevelMax=%s' % USE_LOGLEVELMAX)
         if self.schedule and self.boot_delay:
-            lines.append('ExecStartPre=-%s/systemd-cron/boot_delay %s' % (LIBDIR, self.boot_delay))
+            if UPTIME is None or self.boot_delay > UPTIME:
+                lines.append('ExecStartPre=-%s %s' % (BOOT_DELAY, self.boot_delay))
         lines.append('ExecStart=%s' % self.execstart)
         if self.environment:
-             lines.append('Environment=%s' % environment_string(self.environment))
+            lines.append('Environment=%s' % environment_string(self.environment))
         lines.append('User=%s' % self.user)
-        if self.standardoutput:
-             lines.append('StandardOutput=%s' % self.standardoutput)
         if self.batch:
-             lines.append('CPUSchedulingPolicy=idle')
-             lines.append('IOSchedulingClass=idle')
+            lines.append('CPUSchedulingPolicy=idle')
+            lines.append('IOSchedulingClass=idle')
 
         return '\n'.join(lines)
 
@@ -518,10 +529,7 @@ class Job:
         else:
             lines.append('OnCalendar=%s' % self.schedule)
         if self.random_delay > 1:
-            if RANDOMIZED_DELAY:
-                lines.append('RandomizedDelaySec=%sm' % self.random_delay)
-            else:
-                lines.append('AccuracySec=%sm' % self.random_delay)
+            lines.append('RandomizedDelaySec=%sm' % self.random_delay)
         if self.persistent:
             lines.append('Persistent=true')
 
@@ -561,20 +569,6 @@ class Job:
             f.write(self.generate_service() + '\n')
 
 
-
-def which(exe:str) -> Optional[str]:
-    '''TODO: we could use the PATH= variable from the crontab'''
-    for path in os.environ.get('PATH', '/usr/bin:/bin').split(os.pathsep):
-        try:
-            abspath = os.path.join(path, exe)
-            statbuf = os.stat(abspath)
-        except:
-            continue
-        if stat.S_IMODE(statbuf.st_mode) & 0o111:
-            return abspath
-
-    return None
-
 def files(dirname:str) -> list[str]:
     try:
         return list(filter(os.path.isfile, [os.path.join(dirname, f) for f in os.listdir(dirname)]))
@@ -583,7 +577,7 @@ def files(dirname:str) -> list[str]:
 
 def environment_string(env:dict[str, str]) -> str:
     line = []
-    for k, v in env.items():
+    for k, v in sorted(env.items()):
         if ' ' in v:
             line.append('"%s=%s"' % (k, v))
         else:
@@ -614,7 +608,7 @@ def parse_crontab(filename:str,
             while '  ' in line:
                 line = line.replace('  ', ' ')
 
-            envvar = envvar_re.match(line)
+            envvar = ENVVAR_RE.match(line)
             if envvar:
                 key = envvar.group(1)
                 value = envvar.group(2)
@@ -717,8 +711,9 @@ def workaround_var_not_mounted():
         if e.errno != errno.EEXIST:
             raise
 
-def is_masked(name:str, distro_mapping:dict[str,str]) -> bool:
+def is_masked(path:str, name:str, distro_mapping:dict[str,str]) -> bool:
     '''check if distribution also provide a native .timer'''
+    fullname = os.path.join(path, name)
     for unit_file in ('/lib/systemd/system/%s.timer' % name,
                       '/etc/systemd/system/%s.timer' % name,
                       '/run/systemd/system/%s.timer' % name):
@@ -728,12 +723,12 @@ def is_masked(name:str, distro_mapping:dict[str,str]) -> bool:
                 reason = 'it is masked'
             else:
                 reason = 'native timer is present'
-            log(Log.NOTICE, 'ignoring %s because %s' % (name, reason))
+            log(Log.NOTICE, 'ignoring %s because %s' % (fullname, reason))
             return True
 
     name_distro = '%s.timer' % distro_mapping.get(name, name)
     if os.path.exists('/lib/systemd/system/%s' % name_distro):
-        log(Log.NOTICE, 'ignoring %s because there is %s' % (name, name_distro))
+        log(Log.NOTICE, 'ignoring %s because there is %s' % (fullname, name_distro))
         return True
 
     return False
@@ -772,10 +767,10 @@ def main() -> None:
     CRONTAB_FILES = files('/etc/cron.d')
     for filename in CRONTAB_FILES:
         basename = os.path.basename(filename)
-        if is_masked(basename, CROND2TIMER):
+        if is_masked('/etc/cron.d', basename, CROND2TIMER):
             continue
         if is_backup(basename):
-            log(Log.DEBUG, 'ignoring %s' % basename)
+            log(Log.DEBUG, 'ignoring %s' % filename)
             continue
         for job in parse_crontab(filename, withuser=True):
             if not job.valid:
@@ -795,14 +790,14 @@ def main() -> None:
             CRONTAB_FILES = files('/etc/cron.' + period)
             for filename in CRONTAB_FILES:
                 basename = os.path.basename(filename)
-                if is_masked(basename, PART2TIMER):
+                if is_masked(directory, basename, PART2TIMER):
                     continue
-                if is_backup(basename):
+                if is_backup(basename) or basename == '0anacron':
                     log(Log.DEBUG, 'ignoring %s' % filename)
                     continue
 
                 job = Job(filename, filename)
-                job.persistent = PERSISTENT
+                job.persistent = True
                 job.period = period
                 job.boot_delay = i * 5
                 job.command = [filename]
@@ -846,10 +841,15 @@ if __name__ == '__main__':
     TARGET_DIR = sys.argv[1]
     TIMERS_DIR = os.path.join(TARGET_DIR, 'cron.target.wants')
 
+    run_by_systemd = len(sys.argv) == 4
+    if run_by_systemd:
+        with open('/proc/uptime', 'r') as f:
+            UPTIME = int(float(f.readline().split()[0])) / 60
+
     try:
         main()
     except Exception as e:
-        if len(sys.argv) == 4:
+        if run_by_systemd:
             with open('/dev/kmsg', 'w') as fd:
                 fd.write('<%s> %s[%s]: global exception: %s\n' % (Log.CRIT, SELF, os.getpid(), e))
             exit(1)
